import keyBy from "lodash/keyBy";
import property from "lodash/property";
import { createAsyncAction, errorResult, successResult } from "pullstate";

import { fetch } from "api";
import { AnnouncementStore } from "stores";
import {
  announcementTypeMappingRev,
  parseRawAnnouncement,
  syncAnnoucementItemIds,
} from "utils";

export const loadAnnouncements = createAsyncAction(
  async () => {
    try {
      const resp = await fetch("/announcements");
      const data = await resp.json();
      return successResult(data.data);
    } catch (err) {
      return errorResult([], err.toString());
    }
  },
  {
    postActionHook: ({ result }) => {
      if (!result.error) {
        const announcements = result.payload.map(parseRawAnnouncement);

        AnnouncementStore.update((state) => {
          state.items = keyBy(announcements, property("id"));
          syncAnnoucementItemIds(state);
        });
      }
    },
  }
);

/**
 * Add new announcement.
 */
export const addAnnouncement = createAsyncAction(
  async ({ content, link, type }) => {
    try {
      const body = JSON.stringify({
        content,
        link,
        type: announcementTypeMappingRev[type],
      });
      const resp = await fetch("/announcements", {
        method: "POST",
        body,
        expectedStatus: 201,
      });
      const data = await resp.json();
      return successResult(data.data);
    } catch (err) {
      return errorResult([], err.toString());
    }
  }
);

/**
 * Delete existing announcement.
 */
export const deleteAnnouncement = createAsyncAction(
  /**
   *
   * @param {CF2021.Announcement} item
   */
  async (item) => {
    try {
      await fetch(`/announcements/${item.id}`, {
        method: "DELETE",
        expectedStatus: 204,
      });
      return successResult({ item });
    } catch (err) {
      return errorResult([], err.toString());
    }
  }
);

/**
 * Update content of an announcement.
 */
export const updateAnnouncementContent = createAsyncAction(
  /**
   *
   * @param {CF2021.Announcement} item
   * @param {string} newContent
   */
  async ({ item, newContent }) => {
    try {
      const body = JSON.stringify({
        content: newContent,
      });
      await fetch(`/announcements/${item.id}`, {
        method: "PUT",
        body,
        expectedStatus: 204,
      });
      return successResult({ item, newContent });
    } catch (err) {
      return errorResult([], err.toString());
    }
  }
);
