import React, { useState } from "react";
import classNames from "classnames";

import { addAnnouncement } from "actions/announcements";
import Button from "components/Button";
import ErrorMessage from "components/ErrorMessage";
import MarkdownEditor from "components/mde/MarkdownEditor";
import { useActionState } from "hooks";

const urlRegex = /https?:\/\/(www\.)?[-a-zA-Z0-9@:%._+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_+.~#?&//=]*)/;

const AddAnnouncementForm = ({ className }) => {
  const [text, setText] = useState("");
  const [link, setLink] = useState("");
  const [linkValid, setLinkValid] = useState(null);
  const [noTextError, setNoTextError] = useState(false);
  const [type, setType] = useState("announcement");

  const [adding, addingError] = useActionState(addAnnouncement, {
    content: text,
    link,
    type,
  });

  const onTextInput = (newText) => {
    setText(newText);

    if (newText !== "") {
      setNoTextError(false);
    }
  };

  const onLinkInput = (evt) => {
    setLink(evt.target.value);

    if (!!evt.target.value) {
      setLinkValid(!!evt.target.value.match(urlRegex));
    }
  };

  const onAdd = async (evt) => {
    if (!link) {
      setLinkValid(false);
    }

    if (!!text) {
      if (type === "voting" && link && linkValid) {
        const result = await addAnnouncement.run({ content: text, link, type });

        if (!result.error) {
          setText("");
          setLink("");
        }
      }
    } else {
      setNoTextError(true);
    }
  };

  return (
    <div className={className}>
      {addingError && (
        <ErrorMessage>
          Při přidávání oznámení došlo k problému: {addingError}.
        </ErrorMessage>
      )}
      <div className="grid grid-cols-1 gap-4">
        <div
          className="form-field"
          onChange={(evt) => setType(evt.target.value)}
        >
          <div className="form-field__wrapper text-sm">
            <div className="radio form-field__control">
              <label>
                <input
                  type="radio"
                  name="type"
                  value="announcement"
                  defaultChecked
                />
                <span>Oznámení</span>
              </label>
            </div>

            <div className="radio form-field__control">
              <label>
                <input type="radio" name="type" value="voting" />
                <span>Rozhodující hlasování</span>
              </label>
            </div>
          </div>
        </div>

        <MarkdownEditor
          value={text}
          onChange={onTextInput}
          error={
            noTextError
              ? "Před přidáním oznámení nezapomeňte vyplnit jeho obsah."
              : null
          }
          placeholder="Vyplňte text oznámení"
          toolbarCommands={[
            ["bold", "italic", "strikethrough"],
            ["link", "unordered-list", "ordered-list"],
          ]}
          minEditorHeight={100}
        />

        <div
          className={classNames("form-field", {
            hidden: type !== "voting",
            "form-field--error": linkValid === false,
          })}
        >
          <div className="form-field__wrapper form-field__wrapper--shadowed">
            <input
              type="text"
              className="text-input text-sm text-input--has-addon-l form-field__control"
              value={link}
              placeholder="URL hlasování"
              onChange={onLinkInput}
            />
            <div className="text-input-addon text-input-addon--l order-first">
              <i className="ico--link1"></i>
            </div>
          </div>
          {linkValid === false && (
            <div className="form-field__error">Zadejte platnou URL.</div>
          )}
        </div>
      </div>

      <Button
        onClick={onAdd}
        className="text-sm mt-4"
        hoverActive
        loading={adding}
        disabled={adding}
      >
        Přidat oznámení
      </Button>
    </div>
  );
};

export default AddAnnouncementForm;
